// brandy.cpp : Defines the entry point for the application.
//

#include "stdafx.h"

int main(char *CmdLine);

#include <windows.h>
#include <commctrl.h>
#include <aygshell.h>
#include "tty.h"
#include "resource.h"

#include "common.h"
#include "target.h"
#include "basicdefs.h"
#include "miscprocs.h"
#include "tokens.h"
#include "statement.h"
#include "variables.h"
#include "editor.h"
#include "errors.h"
#include "heap.h"
#include "stack.h"
#include "strings.h"
#include "evaluate.h"
#include "screen.h"
#include "keyboard.h"
#include "mainstate.h"

HWND hMainWnd,                          // Main window handle
     hTermWnd,                          // Terminal window handle
     hCmdBarWnd;                        // The command bar window handle

HINSTANCE hInst;                        // hInstance of the application
TCHAR szTitle[] = TEXT("Brandy");          // Main window name
TCHAR szAppName[] = TEXT("Brandy BBC Basic");
                                        // Main window class name
HFONT hFont = NULL;                     // Handle to the font to use

TERM_SCREEN Screen;                     

char CmdLine[256];

#define MENU_HEIGHT       26
#define BUTTONBAR_HEIGHT  0            // Height of the check box bar
#define BUTTON_HEIGHT     0            // Height of the check boxes


void push_key(int ch);


/***********************************************************************

FUNCTION: 
  TermWndProc

PURPOSE: 
  Processes messages sent to the terminal window.

***********************************************************************/
LRESULT CALLBACK TermWndProc (HWND hWnd, UINT uMsg, WPARAM wParam,
                              LPARAM lParam)
{
  HDC hDC;
  PAINTSTRUCT ps;

  switch (uMsg)
  {
    case WM_SETFOCUS:
      Screen.bHaveFocus = TRUE;
      CreateCaret (hWnd, NULL, Screen.CharWidth, Screen.CharHeight);

      if (ON_SCREEN (Screen.CursorRow, Screen.CursorCol))
      {
        SetCaretPos (COLX(Screen.CursorCol), ROWY(Screen.CursorRow));
        ShowCaret (hWnd);    
      }
      break;

    case WM_KILLFOCUS:
      Screen.bHaveFocus = FALSE;
      HideCaret (hWnd);
      DestroyCaret ();
      break;

    case WM_SIZE:
      // Resize everything.
      InitScreenSettings (hWnd);
      break;

    case WM_PAINT:
      HideCaret (hTermWnd);
      hDC = BeginPaint (hWnd, &ps);
      PaintScreen (hWnd, hDC, &(ps.rcPaint));
      EndPaint (hWnd, &ps);
      ShowCaret (hTermWnd);
      break;

    case WM_CHAR:
		push_key((BYTE)(wParam & 0xFF));
      break;

    case WM_VSCROLL:
      switch (LOWORD(wParam))
      {
        case SB_LINEUP:
          ScrollDown (-1);
          break;

        case SB_LINEDOWN:
          ScrollDown (1);
          break;

        case SB_PAGEUP:
          ScrollDown (-(Screen.ScreenRows - 1));
          break;

        case SB_PAGEDOWN:
          ScrollDown (Screen.ScreenRows - 1);
          break;

        case SB_TOP:
          ScrollDown (-Screen.NumRows);
          break;

        case SB_BOTTOM:
          ScrollDown (Screen.NumRows);
          break;

        case SB_THUMBPOSITION:
          ScrollDown (HIWORD(wParam) - Screen.TopRow);
          break;

        case SB_THUMBTRACK:
          break;

        case SB_ENDSCROLL:
          break;
      }
      break;

    case WM_HSCROLL:
      switch (LOWORD(wParam))
      {
        case SB_LINELEFT:
          ScrollRight (-1);
          break;

        case SB_LINERIGHT:
          ScrollRight (1);
          break;

        case SB_PAGELEFT:
          ScrollRight (-(Screen.ScreenCols - 1));
          break;

        case SB_PAGERIGHT:
          ScrollRight (Screen.ScreenCols - 1);
          break;

        case SB_LEFT:
          ScrollRight (-Screen.NumCols);
          break;

        case SB_RIGHT:
          ScrollRight (Screen.NumCols);
          break;

        case SB_THUMBPOSITION:
          ScrollRight (HIWORD(wParam) - Screen.LeftCol);
          break;

        case SB_THUMBTRACK:
          break;

        case SB_ENDSCROLL:
          break;
      }
      break;

    case WM_DESTROY:
      PostQuitMessage (0);
      break;
  }

  return (DefWindowProc (hWnd, uMsg, wParam, lParam));
}


/***********************************************************************

FUNCTION: 
  MainWndProc

PURPOSE: 
  Processes messages sent to the main window.
      
***********************************************************************/
LRESULT CALLBACK MainWndProc (HWND hWnd, UINT uMsg, WPARAM wParam,
                              LPARAM lParam)
{
  HDC hDC;
  RECT rect;
  HBRUSH hBrush;
  PAINTSTRUCT ps;

  switch (uMsg)
  {
    case WM_CREATE:
    {
      SHMENUBARINFO mbi;

      memset (&mbi, 0, sizeof (SHMENUBARINFO));
      mbi.cbSize     = sizeof (SHMENUBARINFO);
      mbi.hwndParent = hWnd;
      mbi.nToolBarId = IDM_MAIN_MENU;
      mbi.hInstRes   = hInst;
      mbi.nBmpId     = 0;
      mbi.cBmpImages = 0;    

      if (!SHCreateMenuBar(&mbi))
      {
        MessageBox(hWnd, L"SHCreateMenuBar Failed", L"Error", MB_OK);
        return -1;
      }
      
      hCmdBarWnd = mbi.hwndMB;
      return 0;
    }

    case WM_PAINT:
      hDC = BeginPaint (hWnd, &ps);

      // Draw the gray background for the button bar.
      hBrush = (HBRUSH)GetStockObject (LTGRAY_BRUSH);
      GetClientRect (hWnd, &rect);
      rect.bottom = BUTTONBAR_HEIGHT;
      FillRect (hDC, &rect, hBrush);
      DeleteObject (hBrush);

      // Draw the black background for rest of the area.
      hBrush = (HBRUSH)GetStockObject (BLACK_BRUSH);
      rect.top = BUTTONBAR_HEIGHT - 1;
      FillRect (hDC, &rect, hBrush);
      DeleteObject (hBrush);

      EndPaint (hWnd, &ps);
      return 0;

    case WM_SETFOCUS:
      // Give the focus to the terminal window.
      SetFocus (hTermWnd);
      return 0;

    case WM_SIZE:
      GetClientRect (hWnd, &rect);
      //rect.top = MENU_HEIGHT;
      //rect.top += BUTTONBAR_HEIGHT;
      SetWindowPos (hTermWnd, HWND_TOP, rect.left, rect.top,
                    rect.right - rect.left, rect.bottom - rect.top, 0);
      ShowWindow (hTermWnd, SW_SHOWNORMAL);
      return 0;

    case WM_COMMAND:
      switch (LOWORD(wParam))
      {
        case ID_FILE_EXIT:
          DestroyWindow (hWnd);
          break;

        case ID_EDIT_CLEARBUFFER:
          ClearScreen ();
          break;

        case ID_ESCAPE:
		  push_key(0x1b);
          break;

        case ID_QUIT:
		  exit_interpreter(0);
		  break;

		case ID_HELP_ABOUTBRANDY:
          MessageBox (hWnd, 
                      TEXT("Brandy BBC Basic\r\n")
                      TEXT("(c) 2006"),
                      TEXT("About Brandy"), 
                      MB_OK);
          break;

        default:
          MessageBox (hWnd, 
                      TEXT("Unhandled WM_COMMAND message"),
                      TEXT("ERROR"), 
                      MB_OK);
          break;
      }
      return 0;

    case WM_DESTROY:
    {

      PostQuitMessage (0);
      return 0;
    }
  }

  return (DefWindowProc (hWnd, uMsg, wParam, lParam));
}

/***********************************************************************

FUNCTION: 
  InitApplication

PURPOSE: 
  Initialize and register window class.

***********************************************************************/
BOOL InitApplication (HINSTANCE hInstance)
{
  WNDCLASS  wc;

  wc.style = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc = (WNDPROC)MainWndProc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = hInstance;
  wc.hIcon = (HICON)NULL;
  wc.hCursor = NULL;
  wc.hbrBackground = (HBRUSH)GetStockObject (WHITE_BRUSH);
  wc.lpszMenuName = 0;
  wc.lpszClassName = szAppName;

  if (!RegisterClass (&wc))
    return FALSE;
  
  wc.style = CS_VREDRAW | CS_HREDRAW;
  wc.lpfnWndProc = (WNDPROC)TermWndProc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = hInstance;
  wc.hIcon = (HICON)NULL;
  wc.hCursor = NULL;
  wc.hbrBackground = (HBRUSH)GetStockObject (WHITE_BRUSH);
  wc.lpszMenuName = 0;
  wc.lpszClassName = TEXT("BrandyClass");

  return RegisterClass (&wc);
}

/***********************************************************************

FUNCTION: 
  InitInstance

PURPOSE: 
  Create and display the main window.

***********************************************************************/
BOOL InitInstance (HINSTANCE hInstance, int nCmdShow)
{
  RECT rect;
  DWORD dwStyle,
        dwError;

  hMainWnd = CreateWindowEx (0, 
                             szAppName,
                             szTitle,
                             WS_CLIPCHILDREN,
                             0,0,
                             CW_USEDEFAULT, 
                             CW_USEDEFAULT,
                             NULL, 
                             NULL,
                             hInstance, 
                             NULL);
  if (!hMainWnd)
  {
    dwError = GetLastError ();
    return FALSE;
  }

  ShowWindow (hMainWnd, nCmdShow);
  UpdateWindow (hMainWnd);

  GetClientRect (hMainWnd, &rect);
  rect.top = MENU_HEIGHT;
  rect.bottom -= 2*BUTTONBAR_HEIGHT;

  // Window style
  dwStyle = WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX;

  
  // Create the following check box for monitoring a set of events.
  // Because these checkboxes are used for monitoring events only, 
  // make all of them disabled upon creation.

  // Window style
  dwStyle = WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX | WS_DISABLED;
  rect.bottom += BUTTONBAR_HEIGHT;

  // Terminal window style
  dwStyle = WS_CHILD | WS_VSCROLL | WS_VISIBLE | WS_BORDER;
  rect.bottom -= BUTTONBAR_HEIGHT;

  // Create the terminal window.
  hTermWnd = CreateWindowEx (0, 
                             TEXT("BrandyClass"),
                             TEXT("BRANDY"),
                             dwStyle,
                             rect.left, 
                             rect.top, 
                             rect.right - rect.left,
                             rect.bottom - rect.top,
                             hMainWnd, 
                             NULL,
                             hInstance, 
                             NULL);
  if (!hTermWnd)
  {
    dwError = GetLastError ();
    return FALSE;
  }

  SetFocus (hTermWnd);

  return TRUE;
}



/***********************************************************************

FUNCTION: 
  WinMain

PURPOSE: 
  The WinMain function of the application. It is called by the system as
  the initial entry point for this WindowsCE-based application.

***********************************************************************/
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, 
                    LPTSTR lpCmdLine, int nCmdShow)
{
  hInst = hInstance;

  if (!hPrevInstance)
  {
    if (!InitApplication (hInstance))
      return 0;
  }

  if (!InitInstance (hInstance, nCmdShow))
  {
    if (hTermWnd)
      DestroyWindow (hTermWnd);
    
    goto ExitMain;
  }

  // Initialize the screen.
  if (!InitScreenSettings (hTermWnd))
  {
    MessageBox (hMainWnd, 
                TEXT("Unable to allocate screen buffer"),
                TEXT("Fatal Error"), 
                MB_OK);

    DestroyWindow (hTermWnd);
    goto ExitMain;
  }

  // Clear the screen.
  ClearScreen ();


  *CmdLine = 0;
  if (*lpCmdLine)
  {
    WideCharToMultiByte(CP_ACP, NULL, lpCmdLine, -1, CmdLine, 256, NULL, NULL);
  }

  main(CmdLine);
  
ExitMain:

  if (hFont)
    DeleteObject ((HGDIOBJ)hFont);
  
  return 1;
}


